package com.jolira.jagent;

import static java.util.logging.Level.FINE;
import static org.objectweb.asm.ClassWriter.COMPUTE_FRAMES;

import java.lang.instrument.ClassFileTransformer;
import java.lang.instrument.IllegalClassFormatException;
import java.security.ProtectionDomain;
import java.util.logging.Logger;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;

final class InetAddressTransformer implements ClassFileTransformer {
    private final static Logger LOG = Logger
            .getLogger(InetAddressTransformer.class.getName());

    @Override
    public byte[] transform(final ClassLoader loader, final String className,
            final Class<?> classBeingRedefined,
            final ProtectionDomain protectionDomain, final byte[] bytes)
            throws IllegalClassFormatException {
        if (!"java/net/InetAddress".equals(className)) {
            if (LOG.isLoggable(FINE)) {
                LOG.fine("Skipping class " + className);
            }

            return null;
        }

        LOG.info("instrumenting class " + className);

        final ClassReader creader = new ClassReader(bytes);
        final ClassWriter cwriter = new ClassWriter(COMPUTE_FRAMES);
        final ClassAdapter adapter = new InetAddressDecorator(cwriter);

        creader.accept(adapter, 0);

        return cwriter.toByteArray();
    }
}
